/*
 *      ncur.c
		Some basic functions for handling ncurses and pthreads engine
		of the game.
		This code is GPL'ed !! See LICENSE
 */

#include "hdr.h"

void initscreen (void)
{
	int x, y;
	initscr();		
	getmaxyx(stdscr,y,x); //wymiary konsoli powinny byc standardowe, poniewaz do takich sa plansze
	if ( x != 80 || y != 24 )	
	{	
		endwin();
		printf("%dx%d Wymaga terminala 24x80. Sprobuj:\n$ xterm -geometry 80x24\n", y, x);
		exit(1);
	}	
	if(has_colors() == FALSE)//uzywamy kolorowej konsoli
	{	
		endwin();
		printf("Program wymaga kolorowego terminala. Prosze sprobowac xterm\n");
		exit(1);
	}
	start_color();	/*inicjalizacja kolorkow tu i ponizej*/
	init_pair(SCIANA, COLOR_BLUE, COLOR_BLUE); //sciany sa niebieskie
	init_pair(LUDEK, COLOR_GREEN, COLOR_BLACK); //ludzik - zielone "o"
	init_pair(BOMBA, COLOR_RED, COLOR_BLACK); //bomba - czerwone "!"
	init_pair(NAGRODA, COLOR_WHITE, COLOR_BLACK); //zarcie=nagroda - biala kropka
	init_pair(OUT, COLOR_YELLOW, COLOR_BLACK); //wyjscie = zolte "A"
	init_pair(TLO, COLOR_BLACK, COLOR_BLACK); //tlo
	init_pair(BUM, COLOR_RED, COLOR_YELLOW); //wybuch	
	init_pair(DIRT, COLOR_YELLOW, COLOR_RED); /* dirt */
	nodelay(stdscr, TRUE); //getch nie blokuje :)	
	curs_set(0); //niewidzialny kursor
	cbreak(); //entery niepotrzebne
	noecho(); //jasne	
	keypad(stdscr, TRUE); //potrzebne do obslugi kursorow
	clear();
}

void wyswietlplansze(plansza* pla)
{
	int i, j;	
	for (j=0; j < 23; j++)
		for (i=0; i < 80; i++)
			mvaddch(j, i, pla->tab[j][i]);
	statusBar(pla->howMuchToEat, 0, pla->lives, pla->curlvl, 1);
	refresh();
}

void statusBar(short int howMuchToEat, short int fanty, short int lives, short int curlvl, short int died)
{
	#define OFFSET 10
	mvprintw(23, OFFSET, "Level:    | Zbierz:    | Masz:    | Zycia:    | Seppuku: F10");
	
	attron(A_BOLD);	
	mvprintw(23, OFFSET+7, "%d", curlvl);
	mvprintw(23, OFFSET+20, "%d", howMuchToEat);
	
	if(fanty >= howMuchToEat)
		attron(COLOR_PAIR(LUDEK));		
	
	mvprintw(23, OFFSET+31, "%d", fanty);	
	
	if(fanty >= howMuchToEat)
		attroff(COLOR_PAIR(LUDEK));
	
	if(died != 0)
	{
		if(lives != 1)
		{
			attron(COLOR_PAIR(LUDEK));	
			mvprintw(23, OFFSET+43, "%d", lives);	
			attroff(COLOR_PAIR(LUDEK));
		}
		else
		{
			attron(COLOR_PAIR(OUT));	
			mvprintw(23, OFFSET+43, "%d", lives);	
			attroff(COLOR_PAIR(OUT));
		}
	}
	else
	{
		attron(COLOR_PAIR(BOMBA));	
		mvprintw(23, OFFSET+43, ";(");	
		attroff(COLOR_PAIR(BOMBA));
	}
	attroff(A_BOLD);	
}

void inicjujplansze(plansza* pla, short int lives, short int curlvl)
{
	int i, j;
	int ktoryzkolei = 0;
	int whichBird = 0;
	
	pla->tabOfGravs = (gravitable*) malloc(pla->numOfGravs*sizeof(gravitable));
	pla->gracz = (ludek*) malloc(sizeof(ludek));
	
	pla->listOfExpls = (explosion*) malloc(sizeof(explosion));
	pla->shownExpls = (explosion*) malloc(sizeof(explosion));
	pla->lastExpl = pla->listOfExpls;
	pla->lastExpl->valid = 1;
	pla->lastExpl->next = NULL;
	pla->shownExpls->valid = 1;
	pla->shownExpls->next = NULL;
	pla->gameflow = 1;
	pla->lives = lives;
	pla->curlvl = curlvl;
		
	for(j=0; j < 23; j++)
		for(i=0; i < 80; i++)
		{	
			if(pla->tab[j][i] == IS_STONE || pla->tab[j][i] == IS_GEM ) /* create next gravitable... */
			{	
				pla->tabOfGravs[ktoryzkolei].numer = ktoryzkolei;
				pla->gravitables[j][i] = ktoryzkolei;
				pla->tabOfGravs[ktoryzkolei].ruch = 1; /* is not marked for moving */
				pla->tabOfGravs[ktoryzkolei].zabija = 1; /* don't kill */
				pla->tabOfGravs[ktoryzkolei].x = i;
				pla->tabOfGravs[ktoryzkolei].y = j;
				ktoryzkolei++;
			}
			else if (pla->tab[j][i] == IS_PLAYER)
			{
				pla->gracz->fanty = 0;
				pla->gracz->x = i; 
				pla->gracz->y = j; 				
			}
			else if (pla->tab[j][i] == IS_BIRD)
			{
				pla->tabOfBirds[whichBird].y = j;
				pla->tabOfBirds[whichBird].x = i;
				pla->tabOfBirds[whichBird].movex = -1;
				pla->tabOfBirds[whichBird].movey = 0;
				whichBird++;
			}
		}	
		
	pla->gravityAsleep = 1;
	pla->playerAsleep = 1;
	pla->mainloopAsleep = 1;
	pla->direction = 1;	
	
	wyswietlplansze(pla);	/* now show the screen. before the loop for performance */	
}

void koniecgry (plansza* pla) /* FIXME */ /* DIRRRTY! */
{
	/* clean memory: */
	freeExplosions(pla);
	free(pla->tabOfGravs);
	free(pla->tabOfBirds);
	free(pla->gracz);
	free(pla);
}
